@@ -29,6 +29,7 @@ from ontobio.io.assocwriter import GafWriter, GpadWriter
 from ontobio.io import assocparser
 from ontobio.io import gaference
 from ontobio.slimmer import get_minimal_subgraph
+import sys
 import json
 import logging
 
@@ -70,6 +71,14 @@ def main():
                         help="Output from Gaferencer run on a set of GAF annotations")
     parser.add_argument('-v', '--verbosity', default=0, action='count',
                         help='Increase output verbosity')
+    parser.add_argument("--allow_paint", required=False, action="store_const", const=True,
+                        help="Allow IBAs in parser")
+    parser.add_argument("-g", "--gpi", type=str, required=False, default=None,
+                        help="GPI file")
+    parser.add_argument("-l", "--rule", action="append", required=None, default=[], dest="rule_set",
+                        help="Set of rules to be run. Default is no rules to be run, with the exception \
+                            of gorule-0000027 and gorule-0000020. See command line documentation in the \
+                                ontobio project or readthedocs for more information")
 
 
     subparsers = parser.add_subparsers(dest='subcommand', help='sub-command help')
@@ -82,8 +91,10 @@ def main():
 
     parser_n = subparsers.add_parser('convert', help='Convert associations')
     parser_n.set_defaults(function=convert_assocs)
-    parser_n.add_argument('-t', '--to', type=str, required=True,
+    parser_n.add_argument('-t', '--to', type=str, required=True, choices=["GAF", "GPAD", "gaf", "gpad"],
                           help='Format to convert to')
+    parser_n.add_argument("-n", "--format-version", dest="version", type=str, required=False, default=None,
+                          help="Version for the file format. GAF default is 2.1, GPAD default is 1.2")
 
     parser_n = subparsers.add_parser('map2slim', help='Map to a subset/slim')
     parser_n.set_defaults(function=map2slim)
@@ -121,6 +132,10 @@ def main():
     if args.gaferencer_file:
         gaferences = gaference.build_annotation_inferences(json.load(args.gaferencer_file))
 
+    rule_set = args.rule_set
+    if rule_set == ["all"]:
+        rule_set = assocparser.RuleSet.ALL
+
     # set configuration
     filtered_evidence_file = open(args.filtered_file, "w") if args.filtered_file else None
     config = assocparser.AssocParserConfig(
@@ -130,7 +145,10 @@ def main():
         entity_idspaces=args.subject_prefix,
         filter_out_evidence=args.filter_out,
         filtered_evidence_file=filtered_evidence_file,
-        annotation_inferences=gaferences
+        annotation_inferences=gaferences,
+        paint=args.allow_paint,
+        gpi_authority_path=args.gpi,
+        rule_set=rule_set
     )
     p = None
     fmt = None
@@ -141,17 +159,15 @@ def main():
 
     # TODO: use a factory
     if fmt == 'gaf':
-        p = GafParser(dataset=args.file)
+        p = GafParser(config=config, dataset=args.file)
     elif fmt == 'gpad':
-        p = GpadParser()
+        p = GpadParser(config=config)
     elif fmt == 'hpoa':
-        p = HpoaParser()
+        p = HpoaParser(config=config)
     elif fmt == "gpi":
         p = entityparser.GpiParser()
         func = validate_entity
 
-    p.config = config
-
     outfh = None
     if args.outfile is not None:
         two_mb = 2097152
@@ -193,15 +209,22 @@ def convert_assocs(ont, file, outfile, p, args):
     write_assocs(assocs, outfile, args)
 
 def write_assocs(assocs, outfile, args):
-    w = GpadWriter()
-    fmt = args.to
+    w = None
+    fmt = args.to.lower()
+    version = args.version
     if fmt is None or fmt == 'gaf':
-        w = GafWriter()
+        if version:
+            w = GafWriter(file=outfile, version=args.version)
+        else:
+            w = GafWriter(file=outfile)
     elif fmt == 'gpad':
-        w = GpadWriter()
+        if version:
+            w = GpadWriter(file=outfile, version=args.version)
+        else:
+            w = GpadWriter(file=outfile)
     else:
         raise ValueError("Not supported: {}".format(fmt))
-    w.file = outfile
+
     w.write(assocs)
 
 def map2slim(ont, file, outfile, p, args):
