@@ -29,6 +29,7 @@ from ontobio.io import gafgpibridge
 from ontobio.io import entitywriter
 from ontobio.io import gaference
 from ontobio.rdfgen import assoc_rdfgen
+from ontobio.rdfgen.gocamgen.gocam_builder import GoCamBuilder, AssocExtractor
 from ontobio.validation import metadata
 from ontobio.validation import tools
 from ontobio.validation import rules
@@ -39,6 +40,7 @@ from typing import Dict, Set
 
 logger = logging.getLogger("ontobio")
 
+
 def thispath():
     os.path.normpath(os.path.abspath(__file__))
 
@@ -99,6 +101,8 @@ def download_a_dataset_source(group, dataset_metadata, target_dir, source_url, b
     # including scheme and such
     reconstructed_url = urllib.parse.urlunsplit((scheme, urllib.parse.urlparse(joined_url).netloc, urllib.parse.urlparse(joined_url).path, source_url_parsed.query, ""))
 
+    click.echo("Using URL `{}`".format(reconstructed_url))
+
     # Using urllib to download if scheme is ftp or file. Otherwise we can use requests and use a progressbar
     if scheme in ["ftp", "file"]:
         urllib.request.urlretrieve(reconstructed_url, path)
@@ -114,6 +118,7 @@ def download_a_dataset_source(group, dataset_metadata, target_dir, source_url, b
 
     return path
 
+
 def download_source_gafs(group_metadata, target_dir, exclusions=[], base_download_url=None, replace_existing_files=True, only_dataset=None):
     """
     This looks at a group metadata dictionary and downloads each GAF source that is not in the exclusions list.
@@ -135,11 +140,9 @@ def download_source_gafs(group_metadata, target_dir, exclusions=[], base_downloa
         # Local target download path setup - path and then directories
         path = download_a_dataset_source(group_metadata["id"], dataset_metadata, target_dir, gaf_url, base_download_url=base_download_url, replace_existing_files=replace_existing_files)
 
-        if dataset_metadata["compression"] == "gzip":
+        if dataset_metadata.get("compression", None) == "gzip":
             # Unzip any downloaded file that has gzip, strip of the gzip extension
-            unzipped = os.path.splitext(path)[0]
-            unzip(path, unzipped)
-            path = unzipped
+            path = unzip_simple(path)
         else:
             # otherwise file is coming in uncompressed. But we want to make sure
             # to zip up the original source also
@@ -149,6 +152,7 @@ def download_source_gafs(group_metadata, target_dir, exclusions=[], base_downloa
 
     return downloaded_paths
 
+
 def check_and_download_mixin_source(mixin_metadata, group_id, dataset, target_dir, base_download_url=None, replace_existing_files=True):
     mixin_dataset = tools.find(mixin_metadata["datasets"], lambda d: d.get("merges_into", "") == dataset)
     if mixin_dataset is None:
@@ -157,10 +161,10 @@ def check_and_download_mixin_source(mixin_metadata, group_id, dataset, target_di
     click.echo("Merging mixin dataset {}".format(mixin_dataset["source"]))
     path = download_a_dataset_source(group_id, mixin_dataset, target_dir, mixin_dataset["source"], base_download_url=base_download_url, replace_existing_files=replace_existing_files)
 
-    unzipped = os.path.splitext(path)[0] # Strip off the .gz extension, leaving just the unzipped filename
-    unzip(path, unzipped)
+    unzipped = unzip_simple(path) # Strip off the .gz extension, leaving just the unzipped filename
     return unzipped
 
+
 def mixin_dataset(mixin_metadata, dataset):
     mixin_dataset_version = tools.find(mixin_metadata["datasets"], lambda d: d.get("merges_into", "") == dataset)
     if mixin_dataset_version is None:
@@ -168,8 +172,10 @@ def mixin_dataset(mixin_metadata, dataset):
 
     return mixin_dataset_version
 
+
 def unzip(path, target):
     click.echo("Unzipping {}".format(path))
+
     def chunk_gen():
         with gzip.open(path, "rb") as p:
             while True:
@@ -184,6 +190,13 @@ def unzip(path, target):
                 tf.write(chunk)
 
 
+def unzip_simple(zipped_path):
+    # 'Simple' meaning no chunking like in unzip()
+    unzipped = os.path.splitext(zipped_path)[0]  # Strip off the .gz extension, leaving just the unzipped filename
+    unzip(zipped_path, unzipped)
+    return unzipped
+
+
 def create_parser(config, group, dataset, format="gaf"):
     if format == "gpad":
         return GpadParser(config=config, group=group, dataset=dataset)
@@ -195,14 +208,15 @@ def create_parser(config, group, dataset, format="gaf"):
 """
 Produce validated gaf using the gaf parser/
 """
+
 @tools.gzips
-def produce_gaf(dataset, source_gaf, ontology_graph, gpipath=None, paint=False, group="unknown", rule_metadata=None, goref_metadata=None, db_entities=None, group_idspace=None, format="gaf", suppress_rule_reporting_tags=[], annotation_inferences=None, group_metadata=None, extensions_constraints=None, rule_contexts=[]):
+def produce_gaf(dataset, source_gaf, ontology_graph, gpipaths=None, paint=False, group="unknown", rule_metadata=None, goref_metadata=None, db_entities=None, group_idspace=None, format="gaf", suppress_rule_reporting_tags=[], annotation_inferences=None, group_metadata=None, extensions_constraints=None, rule_contexts=[], gaf_output_version="2.2", rule_set=assocparser.RuleSet.ALL):
     filtered_associations = open(os.path.join(os.path.split(source_gaf)[0], "{}_noiea.gaf".format(dataset)), "w")
     config = assocparser.AssocParserConfig(
         ontology=ontology_graph,
         filter_out_evidence=["IEA"],
         filtered_evidence_file=filtered_associations,
-        gpi_authority_path=gpipath,
+        gpi_authority_path=gpipaths,
         paint=paint,
         rule_metadata=rule_metadata,
         goref_metadata=goref_metadata,
@@ -212,14 +226,15 @@ def produce_gaf(dataset, source_gaf, ontology_graph, gpipath=None, paint=False,
         annotation_inferences=annotation_inferences,
         group_metadata=group_metadata,
         extensions_constraints=extensions_constraints,
-        rule_contexts=rule_contexts
+        rule_contexts=rule_contexts,
+        rule_set=rule_set,
     )
     logger.info("Producing {}".format(source_gaf))
-    logger.info("AssocParserConfig used: {}".format(config))
+    # logger.info("AssocParserConfig used: {}".format(config))
     split_source = os.path.split(source_gaf)[0]
     validated_gaf_path = os.path.join(split_source, "{}_valid.gaf".format(dataset))
     outfile = open(validated_gaf_path, "w")
-    gafwriter = GafWriter(file=outfile, source=dataset)
+    gafwriter = GafWriter(file=outfile, source=dataset, version=gaf_output_version)
 
     click.echo("Validating source {}: {}".format(format, source_gaf))
     parser = create_parser(config, group, dataset, format)
@@ -262,11 +277,6 @@ def produce_gaf(dataset, source_gaf, ontology_graph, gpipath=None, paint=False,
 
 @tools.gzips
 def make_products(dataset, target_dir, gaf_path, products, ontology_graph):
-    gafparser = GafParser()
-    gafparser.config = assocparser.AssocParserConfig(
-        ontology=ontology_graph,
-        paint=True
-    )
 
     with open(gaf_path) as sg:
         lines = sum(1 for line in sg)
@@ -282,13 +292,17 @@ def make_products(dataset, target_dir, gaf_path, products, ontology_graph):
 
     # def write_gpi_entity(association, bridge, gpiwriter):
     with open(gaf_path) as gf:
+        gafparser = GafParser(config=assocparser.AssocParserConfig(
+            ontology=ontology_graph,
+            paint=True,
+        ))
+
         # gpi info:
         click.echo("Using {} as the gaf to build data products with".format(gaf_path))
         if products["ttl"]:
             click.echo("Setting up {}".format(product_files["ttl"].name))
             rdf_writer = assoc_rdfgen.TurtleRdfWriter(label=os.path.split(product_files["ttl"].name)[1] )
             transformer = assoc_rdfgen.CamRdfTransform(writer=rdf_writer)
-            parser_config = assocparser.AssocParserConfig(ontology=ontology_graph)
 
         if products["gpad"]:
             click.echo("Setting up {}".format(product_files["gpad"].name))
@@ -298,9 +312,8 @@ def make_products(dataset, target_dir, gaf_path, products, ontology_graph):
         with click.progressbar(iterable=gafparser.association_generator(file=gf), length=lines) as associations:
             for association in associations:
                 if products["ttl"]:
-                    if "header" not in association or not association["header"]:
-                        transformer.provenance()
-                        transformer.translate(association)
+                    transformer.provenance()
+                    transformer.translate(association)
 
                 if products["gpad"]:
                     gpadwriter.write_assoc(association)
@@ -430,7 +443,8 @@ def merge_all_mixin_gaf_into_mod_gaf(valid_gaf_path, mixin_gaf_paths):
 
     return merged_path
 
-def mixin_a_dataset(valid_gaf, mixin_metadata_list, group_id, dataset, target, ontology, gpipath=None, base_download_url=None, rule_metadata={}, replace_existing_files=True, rule_contexts=[]):
+
+def mixin_a_dataset(valid_gaf, mixin_metadata_list, group_id, dataset, target, ontology, gpipaths=None, base_download_url=None, rule_metadata={}, replace_existing_files=True, rule_contexts=[], gaf_output_version="2.2"):
 
     end_gaf = valid_gaf
     mixin_gaf_paths = []
@@ -442,7 +456,7 @@ def mixin_a_dataset(valid_gaf, mixin_metadata_list, group_id, dataset, target, o
             mixin_dataset_id = mixin_dataset_metadata["dataset"]
             format = mixin_dataset_metadata["type"]
             context = ["import"] if mixin_metadata.get("import", False) else []
-            mixin_gaf = produce_gaf(mixin_dataset_id, mixin_src, ontology, gpipath=gpipath, paint=True, group=mixin_metadata["id"], rule_metadata=rule_metadata, format=format, rule_contexts=context)[0]
+            mixin_gaf = produce_gaf(mixin_dataset_id, mixin_src, ontology, gpipaths=gpipaths, paint=True, group=mixin_metadata["id"], rule_metadata=rule_metadata, format=format, rule_contexts=context, gaf_output_version=gaf_output_version)[0]
             mixin_gaf_paths.append(mixin_gaf)
 
     if mixin_gaf_paths:
@@ -476,7 +490,9 @@ def cli(ctx, verbose):
 @click.option("--skip-existing-files", "-K", is_flag=True, default=False, help="When downloading files, if a file already exists it won't downloaded over")
 @click.option("--gaferencer-file", "-I", type=click.Path(exists=True), default=None, required=False, help="Path to Gaferencer output to be used for inferences")
 @click.option("--only-dataset", default=None)
-def produce(ctx, group, metadata_dir, gpad, ttl, target, ontology, exclude, base_download_url, suppress_rule_reporting_tag, skip_existing_files, gaferencer_file, only_dataset):
+@click.option("--gaf-output-version", default="2.2", type=click.Choice(["2.1", "2.2"]))
+@click.option("--rule-set", "-l", "rule_set", default=[assocparser.RuleSet.ALL], multiple=True)
+def produce(ctx, group, metadata_dir, gpad, ttl, target, ontology, exclude, base_download_url, suppress_rule_reporting_tag, skip_existing_files, gaferencer_file, only_dataset, gaf_output_version, rule_set):
 
     logger.info("Logging is verbose")
     products = {
@@ -516,10 +532,15 @@ def produce(ctx, group, metadata_dir, gpad, ttl, target, ontology, exclude, base
     if gaferencer_file:
         gaferences = gaference.load_gaferencer_inferences_from_file(gaferencer_file)
 
+    # Default comes through as single-element tuple
+    if rule_set == (assocparser.RuleSet.ALL,):
+        rule_set = assocparser.RuleSet.ALL
+
     for dataset_metadata, source_gaf in downloaded_gaf_sources:
         dataset = dataset_metadata["dataset"]
         # Set paint to True when the group is "paint".
-        # This will prevent filtering of IBA (GO_RULE:26) when paint is being treated as a top level group, like for paint_other.
+        # This will prevent filtering of IBA (GO_RULE:26) when paint is being treated as a top level group,
+        # like for paint_other.
         valid_gaf = produce_gaf(dataset, source_gaf, ontology_graph,
             paint=(group=="paint"),
             group=group,
@@ -531,15 +552,72 @@ def produce(ctx, group, metadata_dir, gpad, ttl, target, ontology, exclude, base
             annotation_inferences=gaferences,
             group_metadata=group_metadata,
             extensions_constraints=extensions_constraints,
-            rule_contexts=["import"] if dataset_metadata.get("import", False) else []
+            rule_contexts=["import"] if dataset_metadata.get("import", False) else [],
+            gaf_output_version=gaf_output_version,
+            rule_set=rule_set
             )[0]
 
         gpi = produce_gpi(dataset, absolute_target, valid_gaf, ontology_graph)
 
-        end_gaf = mixin_a_dataset(valid_gaf, mixin_metadata_list, group_metadata["id"], dataset, absolute_target, ontology_graph, gpipath=gpi, base_download_url=base_download_url, rule_metadata=rule_metadata, replace_existing_files=not skip_existing_files)
+        gpi_list = [gpi]
+        # Try to find other GPIs in metadata and merge
+        for ds in group_metadata["datasets"]:
+            # Where type=GPI for the same dataset (e.g. "zfin", "goa_cow")
+            if ds["type"] == "gpi" and ds["dataset"] == dataset and ds.get("source"):
+                matching_gpi_path = download_a_dataset_source(group, ds, absolute_target, ds["source"],
+                                                              replace_existing_files=not skip_existing_files)
+                if ds.get("compression", None) == "gzip":
+                    matching_gpi_path = unzip_simple(matching_gpi_path)
+                gpi_list.append(matching_gpi_path)
+
+        end_gaf = mixin_a_dataset(valid_gaf, mixin_metadata_list, group_metadata["id"], dataset, absolute_target,
+                                  ontology_graph, gpipaths=gpi_list, base_download_url=base_download_url,
+                                  rule_metadata=rule_metadata, replace_existing_files=not skip_existing_files,
+                                  gaf_output_version=gaf_output_version)
         make_products(dataset, absolute_target, end_gaf, products, ontology_graph)
 
 
+@cli.command()
+@click.pass_context
+@click.option("--gpad_path", "-g", type=click.Path(), required=True)
+@click.option("--gpi_path", "-i", type=click.Path(), required=True)
+@click.option("--target", "-t", type=click.Path(), required=True)
+@click.option("--ontology", "-o", type=click.Path(exists=True), required=True, multiple=True)
+@click.option("--ttl", default=False, is_flag=True)
+@click.option("--modelstate", "-s", default=None)
+def gpad2gocams(ctx, gpad_path, gpi_path, target, ontology, ttl, modelstate):
+    # NOTE: Validation on GPAD not included here since it's currently baked into produce() above.
+    # Multi-param to accept multiple ontology files, then merge to one (this will make a much smaller ontology
+    #  with only what we need, i.e. GO, RO, GOREL)
+    ontology_graph = OntologyFactory().create(ontology[0], ignore_cache=True)
+    for ont in ontology[1:]:
+        ontology_graph.merge([OntologyFactory().create(ont, ignore_cache=True)])
+    parser_config = assocparser.AssocParserConfig(ontology=ontology_graph,
+                                                  gpi_authority_path=gpi_path
+                                                  )
+    extractor = AssocExtractor(gpad_path, parser_config=parser_config)
+    assocs_by_gene = extractor.group_assocs()
+
+    absolute_target = os.path.abspath(target)
+    gpad_basename = os.path.basename(gpad_path)
+    gpad_basename_root, gpad_ext = os.path.splitext(gpad_basename)
+    output_basename = "{}.nq".format(gpad_basename_root)
+    report_basename = "{}.gocamgen.report".format(gpad_basename_root)
+    output_path = os.path.join(absolute_target, output_basename)
+    report_path = os.path.join(absolute_target, report_basename)
+
+    builder = GoCamBuilder(parser_config=parser_config, modelstate=modelstate)
+
+    for gene, associations in assocs_by_gene.items():
+        if ttl:
+            builder.make_model_and_write_out(gene, annotations=associations, output_directory=absolute_target)
+        else:
+            builder.make_model_and_add_to_store(gene, annotations=associations)
+    if not ttl:
+        builder.write_out_store_to_nquads(filepath=output_path)
+
+    builder.write_report(report_filepath=report_path)
+
 
 @cli.command()
 @click.argument("group")
@@ -566,7 +644,8 @@ def paint(group, dataset, metadata, target, ontology):
 @click.option("--metadata", "-m", "metadata_dir", type=click.Path(), required=True)
 @click.option("--out", type=click.Path(), required=False)
 @click.option("--ontology", type=click.Path(), required=True)
-@click.option("--gaferencer-file", "-I", type=click.Path(exists=True), default=None, required=False, help="Path to Gaferencer output to be used for inferences")
+@click.option("--gaferencer-file", "-I", type=click.Path(exists=True), default=None, required=False,
+              help="Path to Gaferencer output to be used for inferences")
 def rule(metadata_dir, out, ontology, gaferencer_file):
     absolute_metadata = os.path.abspath(metadata_dir)
 
@@ -590,7 +669,8 @@ def rule(metadata_dir, out, ontology, gaferencer_file):
         goref_metadata=goref_metadata,
         entity_idspaces=db_entities,
         group_idspace=group_ids,
-        annotation_inferences=gaferences
+        annotation_inferences=gaferences,
+        rule_set=assocparser.RuleSet.ALL
     )
     all_examples_valid = True
     all_results = []
