@@ -85,6 +85,7 @@ class GolrFields:
     OBJECT_TAXON_CLOSURE_LABEL_SEARCHABLE = 'object_taxon_closure_label_searchable'
     SUBJECT_GENE_CLOSURE='subject_gene_closure'
     SUBJECT_GENE_LABEL_SEARCHABLE='subject_gene_label_searchable'
+    OBJECT_GENE_LABEL_SEARCHABLE = 'object_gene_label_searchable'
     SUBJECT='subject'
     SUBJECT_LABEL='subject_label'
     SUBJECT_CLOSURE_LABEL_SEARCHABLE='subject_closure_label_searchable'
@@ -346,6 +347,8 @@ class GolrSearchQuery(GolrAbstractQuery):
                  taxon=None,
                  min_match=None,
                  minimal_tokenizer=False,
+                 include_eqs=False,
+                 exclude_groups=False,
                  user_agent=None):
         self.term = term
         self.category = category
@@ -372,6 +375,8 @@ class GolrSearchQuery(GolrAbstractQuery):
         self.highlight_class = highlight_class
         self.taxon = taxon
         self.min_match = min_match
+        self.include_eqs = include_eqs
+        self.exclude_groups = exclude_groups
         self.minimal_tokenizer = minimal_tokenizer
 
         self.user_agent = get_user_agent(modules=[requests, pysolr], caller_name=__name__)
@@ -489,18 +494,35 @@ class GolrSearchQuery(GolrAbstractQuery):
             params['fq'].append(fq)
 
         if self.prefix is not None:
-            negative_filter = [p_filt for p_filt in self.prefix
+            negative_filter = [p_filt[1:] for p_filt in self.prefix
                                if p_filt.startswith('-')]
             positive_filter = [p_filt for p_filt in self.prefix
                                if not p_filt.startswith('-')]
 
             if negative_filter:
-                neg_filter = '({})'.format(
-                    " AND ".join([filt for filt in negative_filter]))
-                params['fq'].append('prefix:{}'.format(neg_filter))
+                if self.include_eqs:
+                    single_filts = [
+                        f'(-prefix:"{prefix}" OR -equivalent_curie:{prefix}\:*)'
+                        for prefix in negative_filter
+                    ]
+                    for filt in single_filts:
+                        params['fq'].append(filt)
+
+                else:
+                    neg_filter = '({})'.format(" OR ".join([filt for filt in negative_filter]))
+                    params['fq'].append('-prefix:{}'.format(solr_quotify(negative_filter)))
 
             if positive_filter:
-                params['fq'].append('prefix:{}'.format(solr_quotify(positive_filter)))
+                if self.include_eqs:
+                    # fq=((prefix:HP OR equivalent_curie:HP) OR (prefix:MONDO OR equivalent_curie:MONDO))
+                    single_filts = [
+                        f'(prefix:"{prefix}" OR equivalent_curie:{prefix}\:*)'
+                        for prefix in positive_filter
+                    ]
+                    pos_filter = '({})'.format(" OR ".join([filt for filt in single_filts]))
+                    params['fq'].append(pos_filter)
+                else:
+                    params['fq'].append('prefix:{}'.format(solr_quotify(positive_filter)))
 
         if self.boost_fx is not None:
             params['bf'] = []
@@ -516,6 +538,9 @@ class GolrSearchQuery(GolrAbstractQuery):
             for tax in self.taxon:
                 params['fq'].append('taxon:"{}"'.format(tax))
 
+        if self.exclude_groups:
+            params['fq'].append('leaf:1')
+
         if self.min_match is not None:
             params['mm'] = self.min_match
 
@@ -601,6 +626,7 @@ class GolrSearchQuery(GolrAbstractQuery):
 
             doc['taxon'] = doc['taxon'] if 'taxon' in doc else ""
             doc['taxon_label'] = doc['taxon_label'] if 'taxon_label' in doc else ""
+            doc['equivalent_curie'] = doc['equivalent_curie'] if 'equivalent_curie' in doc else []
             doc = AutocompleteResult(
                 id=doc['id'],
                 label=doc['label'],
@@ -609,7 +635,8 @@ class GolrSearchQuery(GolrAbstractQuery):
                 taxon=doc['taxon'],
                 taxon_label=doc['taxon_label'],
                 highlight=hl.highlight,
-                has_highlight=hl.has_highlight
+                has_highlight=hl.has_highlight,
+                equivalent_ids=doc['equivalent_curie']
             )
             docs.append(doc)
 
@@ -970,6 +997,10 @@ class GolrAssociationQuery(GolrAbstractQuery):
                     M.OBJECT_CLOSURE
                 ]
 
+        if self.sort is None and not self._use_amigo_schema(object_category):
+            # Make default descending by count of publications for monarch
+            self.sort = 'source_count desc'
+
         if self.solr is None:
             if self.url is None:
                 endpoint = self.get_config().solr_assocs
@@ -1319,6 +1350,17 @@ class GolrAssociationQuery(GolrAbstractQuery):
         for field in self.non_null_fields:
             filter_queries.append(field + ":['' TO *]")
 
+        search_fields = None
+        if self.q is not None and not self._use_amigo_schema(object_category):
+            search_fields = [
+                M.SUBJECT_LABEL_SEARCHABLE,
+                M.OBJECT_LABEL_SEARCHABLE,
+                M.SUBJECT_TAXON_LABEL_SEARCHABLE,
+                M.OBJECT_TAXON_LABEL_SEARCHABLE,
+                M.SUBJECT_GENE_LABEL_SEARCHABLE,
+                M.OBJECT_GENE_LABEL_SEARCHABLE,
+            ]
+
         params = {
             'q': qstr,
             'fq': filter_queries,
@@ -1327,7 +1369,8 @@ class GolrAssociationQuery(GolrAbstractQuery):
             'facet.limit': facet_limit,
             'facet.mincount': self.facet_mincount,
             'fl': ",".join(list(filter(None, select_fields))),
-            'rows': rows
+            'rows': rows,
+            "defType": "edismax"
         }
 
         if self.start is not None:
@@ -1354,6 +1397,9 @@ class GolrAssociationQuery(GolrAbstractQuery):
         if self.sort is not None:
             params['sort'] = self.sort
 
+        if search_fields:
+            params['qf'] = search_fields
+
         return params
 
     def exec(self, **kwargs):
@@ -1839,4 +1885,4 @@ def replace(json, field, old, new):
     for i in range(0, len(json)):
         if json[i][field]:
             json[i][field] = json[i][field].replace(old, new)
-    return json
\ No newline at end of file
+    return json
