@@ -7,6 +7,7 @@ All parser objects instantiate a subclass of the abstract `AssocParser` object
 
 # TODO: Refactor - move some stuff out into generic parser object
 
+# from _typeshed import NoneType
 import re
 import requests
 import tempfile
@@ -21,13 +22,16 @@ import dateutil.parser
 from dataclasses import dataclass
 
 from collections import namedtuple, defaultdict
-from typing import Optional, List, Dict, Set
+from typing import Callable, ClassVar, Collection, Iterable, Optional, List, Dict, Set, TypeVar, Union, Any
 
 from ontobio import ontol
 from ontobio import ecomap
 from ontobio.io import parsereport
 from ontobio.util.user_agent import get_user_agent
 from ontobio.model import association
+from ontobio.rdfgen import relations
+
+from prefixcommons import curie_util
 
 TAXON = 'TAXON'
 ENTITY = 'ENTITY'
@@ -44,17 +48,170 @@ def write_to_file(optional_file, text):
 
 SplitLine = namedtuple("SplitLine", ["line", "values", "taxon"])
 SplitLine.__str__ = lambda self: self.line
-
 """
 This is a collection that views a gaf line in two ways: as the full line, and
 as the separated by tab list of values. We also tack on the taxon.
 """
 
+@dataclass
+class ValidateResult:
+    valid: bool
+    original: str
+    parsed: Optional[Any]
+    message: str
+
+@dataclass
+class ColumnValidator:
+
+    def cardinality(self, field: str, entity: str, minimum: int, maximum: Optional[int]) -> ValidateResult:
+        items = [i for i in entity.split("|") if i != ""]
+        if len(items) == 0 and 0 < minimum:
+            return ValidateResult(False, "EMPTY", None, "{} must have at least {} items".format(field, minimum))
+        if len(items) < minimum:
+            return ValidateResult(False, entity, None, "{} must have at least {} items".format(field, minimum))
+        elif maximum is not None and len(items) > maximum:
+            return ValidateResult(False, entity, None, "{} cannot have more than {} items".format(field, maximum))
+
+        return ValidateResult(True, entity, items, "")
+
+    def validate(self, entity) -> ValidateResult:
+        return ValidateResult(True, entity, entity, "")
+
+@dataclass
+class Qualifier2_1(ColumnValidator):
+    valid_qualifiers = {
+        "contributes_to",
+        "colocalizes_with",
+    }
+
+    def validate(self, entity) -> ValidateResult:
+        result = self.cardinality("Qualifier", entity, 0, 2)
+        if not result.valid:
+            return result
+
+        invalid_non_not = [q for q in result.parsed if q not in self.valid_qualifiers and q != "NOT"]
+        if invalid_non_not:
+            result.valid = False
+            result.parsed = None
+            result.message = "Invalid Qualifiers: {}".format(", ".join(invalid_non_not))
+
+        return result
+
+@dataclass
+class Qualifier2_2(ColumnValidator):
+
+    valid_qualifiers = {
+        "enables",
+        "contributes_to",
+        "involved_in",
+        "acts_upstream_of",
+        "acts_upstream_of_positive_effect",
+        "acts_upstream_of_negative_effect",
+        "acts_upstream_of_or_within",
+        "acts_upstream_of_or_within_positive_effect",
+        "acts_upstream_of_or_within_negative_effect",
+        "located_in",
+        "part_of",
+        "is_active_in",
+        "colocalizes_with"
+    }
+
+    def validate(self, entity) -> ValidateResult:
+        result = self.cardinality("Qualifier", entity, 1, 2)
+        if not result.valid:
+            return result
+
+        invalid_non_not = [q for q in result.parsed if q not in self.valid_qualifiers and q != "NOT"]
+        if invalid_non_not:
+            result.valid = False
+            result.parsed = None
+            result.message = "Invalid Qualifiers: {}".format(", ".join(invalid_non_not))
+            return result
+
+        if "NOT" in result.parsed and len(result.parsed) == 1:
+            # If not is the only elemnent, then it's wrong
+            result.valid = False
+            result.parsed = None
+            result.message = "`NOT` is not a Qualifier and so cannot exist by itself"
+            return result
+
+        if len(result.parsed) == 2 and "NOT" not in result.parsed:
+            result.valid = False
+            result.parsed = None
+            result.message = "There can be only one relation entry for the Qualifier field"
+            return result
+
+        return result
+
+
+@dataclass
+class CurieValidator(ColumnValidator):
+    def validate(self, entity):
+        curie = association.Curie.from_str(entity)
+        if isinstance(curie, association.Error):
+            return ValidateResult(False, entity, None, curie.info)
+
+        return ValidateResult(True, entity, curie, "")
+
+
+@dataclass
+class TaxonValidator(CurieValidator):
+
+    def validate(self, entity):
+        cardinality_result = self.cardinality("Taxon", entity, 1, 2)
+        if not cardinality_result.valid:
+            return cardinality_result
+
+        parsed_taxons = []
+        for taxa in cardinality_result.parsed:
+            result = super().validate(taxa)
+            if not result.valid:
+                return result
+
+            result.parsed.namespace = "NCBITaxon"
+            parsed_taxons.append(result.parsed)
+
+        return ValidateResult(True, entity, parsed_taxons, "")
+
+@dataclass
+class RuleSet:
+    """
+    This represents the set of GO Rules we want ontobio to run during parsing of annotations.
+
+    If `rules` is None, all rules will be run. If `rules` is an empty set, no rules will be run.
+    Otherwise, the rules set member variable is a set of integers corresponding to the rule IDs
+    that will be run during parse.
+
+    For example, if self.rules == set([6, 13, 20]) then only GORULE:0000006, GORULE:0000013, and 
+    GORULE:0000020 will be run.
+
+    Note that GORULE:0000001 will always be run, as that rule represents succesful parsing a line
+    into ontobio at all.
+    """
+    rules: Optional[Set[int]]
+
+    ALL: ClassVar[str] = "all"
+
+    def __init__(self, rules: Optional[Iterable]):
+        if rules is None:
+            self.rules = None
+        else:
+            self.rules = set(rules)
+    
+    def should_run_rule(self, rule: int) -> bool:
+        if self.rules is None:
+            return True
+        
+        return rule in self.rules
+
+
 class AssocParserConfig():
     """
     Configuration for an association parser
 
     rule_metadata: Dictionary of rule IDs to metadata pulled out by yamldown
+
+    rule_sets: an Iterable of integers representing 
     """
     def __init__(self,
                  remove_double_prefixes=False,
@@ -79,7 +236,8 @@ class AssocParserConfig():
                  suppress_rule_reporting_tags=[],
                  annotation_inferences=None,
                  extensions_constraints=None,
-                 rule_contexts=[]):
+                 rule_contexts=[],
+                 rule_set=None):
 
         self.remove_double_prefixes=remove_double_prefixes
         self.ontology=ontology
@@ -103,6 +261,17 @@ class AssocParserConfig():
         self.extensions_constraints = AssocParserConfig._compute_constraint_subclasses(extensions_constraints, ontology)
         self.group_idspace = None if group_idspace is None else set(group_idspace)
         self.rule_contexts = rule_contexts
+        # We'll say that the default None should run no rules, so let's set the rule_set to []
+        # print("Rule Set is {}".format(rule_set))
+        if rule_set == None:
+            self.rule_set = RuleSet([])
+        elif rule_set == RuleSet.ALL:
+            # None here means all rules
+            self.rule_set = RuleSet(None)
+        else:
+            self.rule_set = RuleSet(rule_set)
+
+
         # This is a dictionary from ruleid: `gorule-0000001` to title strings
         if self.exclude_relations is None:
             self.exclude_relations = []
@@ -142,7 +311,6 @@ class AssocParserConfig():
         attribute_values = ["{att}={val}".format(att=att, val=dict([(k, v) for k, v in value.items()][:8]) if isinstance(value, dict) else value) for att, value in vars(self).items()]
         return "AssocParserConfig({})".format(",".join(attribute_values))
 
-
 class Report(object):
     """
     A report object that is generated as a result of a parse
@@ -175,6 +343,7 @@ class Report(object):
     EXTENSION_SYNTAX_ERROR = "Syntax error in annotation extension field"
     VIOLATES_GO_RULE = "Violates GO Rule"
     RULE_PASS = "Passes GO Rule"
+    INVALID_REFERENCES = "Only one reference per ID space allowed"
 
 
     def __init__(self, group="unknown", dataset="unknown", config=None):
@@ -188,13 +357,13 @@ class Report(object):
         self.config = config
         self.header = []
 
-    def error(self, line, type, obj, msg="", taxon="", rule=None):
+    def error(self, line, type, obj, msg="", taxon: str = "", rule=None):
         self.message(self.ERROR, line, type, obj, msg, taxon=taxon, rule=rule)
 
-    def warning(self, line, type, obj, msg="", taxon="", rule=None):
+    def warning(self, line, type, obj, msg="", taxon: str = "", rule=None):
         self.message(self.WARNING, line, type, obj, msg, taxon=taxon, rule=rule)
 
-    def message(self, level, line, type, obj, msg="", taxon="", rule=None, dont_record=["INFO"]):
+    def message(self, level, line, type, obj, msg="", taxon: str = "", rule=None, dont_record=["INFO"]):
         message = {
             'level': level,
             'line': line,
@@ -210,13 +379,13 @@ class Report(object):
 
         self.reporter.message(message, rule)
 
-
     def add_associations(self, associations):
         for a in associations:
             self.add_association(a)
 
     def add_association(self, association):
-        if "header" in association and association["header"]:
+        if isinstance(association, dict) and "header" in association:
+            # Then we are a header
             self.header.append(association["line"])
         else:
             self.n_assocs += 1
@@ -309,9 +478,6 @@ class ParseResult:
 
 # TODO avoid using names that are builtin python: file, id
 
-parser_version_regex = re.compile(r"!([\w]+)-version:[\s]*([\d]+\.[\d]+(\.[\d]+)?)")
-
-
 class AssocParser(object):
     """
     Abstract superclass of all association parser classes
@@ -356,7 +522,7 @@ class AssocParser(object):
             self.report.report_parsed_result(parsed_result, outfile, self.config.filtered_evidence_file, self.config.filter_out_evidence)
             for association in parsed_result.associations:
                 # yield association if we don't care if it's a header or if it's definitely a real gaf line
-                if not skipheader or "header" not in association:
+                if not skipheader or not isinstance(association, dict):
                     yield association
 
         logger.info(self.report.short_summary())
@@ -598,6 +764,16 @@ class AssocParser(object):
 
         return sorted(valids)
 
+    def validate_curie_ids(self, ids: List[association.Curie], line: SplitLine) -> Optional[List[association.Curie]]:
+        valids = []
+        for i in ids:
+            if self._validate_id(str(i), line):
+                valids.append(i)
+            else:
+                return None
+
+        return valids
+
         # We are only reporting, so just pass it through
 
     def _normalize_id(self, id):
@@ -698,26 +874,73 @@ class AssocParser(object):
 
 
 
-def _normalize_gaf_date(date, report, taxon, line):
-    if date is None or date == "":
-        report.warning(line, Report.INVALID_DATE, date, "GORULE:0000001: empty",
-            taxon=taxon, rule=1)
-        return date
+def parse_date(date: str, report: Report, line: List) -> Optional[association.Date]:
+    if date == "":
+        report.error(line, Report.INVALID_DATE, "\'\'", "GORULE:0000001: empty", rule=1)
+        return None
 
-    # We check int(date)
+    d = None
     if len(date) == 8 and date.isdigit():
-        d = datetime.datetime(int(date[0:4]), int(date[4:6]), int(date[6:8]), 0, 0, 0, 0)
+        # For GAF date, should be YYYYMMDD all as digits and
+        # a well formed date string here will be exactly 8 characters long
+        d = association.Date(year=date[0:4], month=date[4:6], day=date[6:8], time="")
     else:
-        report.warning(line, Report.INVALID_DATE, date, "GORULE:0000001: Date field must be YYYYMMDD, got: {}".format(date),
-            taxon=taxon, rule=1)
+        report.warning(line, report.INVALID_DATE, date, "GORULE:0000001: Date field must be YYYYMMDD, got: {}".format(date), rule=1)
+        parsed = None
         try:
-            d = dateutil.parser.parse(date)
+            parsed = dateutil.parser.parse(date)
         except:
-            report.error(line, Report.INVALID_DATE, date, "GORULE:0000001: Could not parse date '{}' at all".format(date),
-                taxon=taxon, rule=1)
+            report.error(line, Report.INVALID_DATE, date, "GORULE:0000001: Could not parse date '{}' at all".format(date), rule=1)
             return None
 
-    return d.strftime("%Y%m%d")
+        d = association.Date(year="{:02d}".format(parsed.year), month="{:02d}".format(parsed.month), day="{:02d}".format(parsed.day), time="")
+
+    return d
+
+def parse_iso_date(date: str, report: Report, line: List) -> Optional[association.Date]:
+
+    def parse_with_dateutil(date: str, repot: Report, line: List) -> Optional[association.Date]:
+        parsed = None
+        try:
+            parsed = dateutil.parser.parse(date)
+        except:
+            report.error(line, Report.INVALID_DATE, date, "GORULE:0000001: Could not parse date '{}' at all".format(date), rule=1)
+            return None
+
+        return association.Date(
+            year="{:02d}".format(parsed.year),
+            month="{:02d}".format(parsed.month),
+            day="{:02d}".format(parsed.day),
+            time=parsed.time().isoformat())
+
+
+    if date == "":
+        report.error(line, Report.INVALID_DATE, "\'\'", "GORULE:0000001: empty", rule=1)
+        return None
+
+    d = None
+    if len(date) >= 10:
+        # For ISO style date, should be YYYY-MM-DD all as digits and
+        # a well formed date string here will be at least 10 characters long.
+        # Optionally, there could be an appended THH:MM
+        year = date[0:4]
+        month = date[5:7]
+        day = date[8:10]
+        time = date[11:]
+        if year.isdigit() and month.isdigit() and day.isdigit():
+            d = association.Date(year=year, month=month, day=day, time=time)
+        else:
+            # If any of year, month, day are not made of digits, then try to parse with dateutil
+            report.warning(line, report.INVALID_DATE, date, "GORULE:0000001: Date field must be YYYY-MM-DD, got: {}".format(date), rule=1)
+            d = parse_with_dateutil(date, report, line)
+
+    else:
+        # If we got the wrong number characters in the date, then fallback to dateutil
+        report.warning(line, report.INVALID_DATE, date, "GORULE:0000001: Date field must be YYYY-MM-DD, got: {}".format(date), rule=1)
+        d = parse_with_dateutil(date, report, line)
+
+    return d
+
 
 ## we generate both qualifier and relation field
 ## Returns: (negated, relation, other_qualifiers)
