@@ -11,7 +11,7 @@ class HpoaParser(AssocParser):
     Note that there are similarities with Gaf format, so we inherit from GafParser, and override
     """
 
-    def __init__(self,config=assocparser.AssocParserConfig()):
+    def __init__(self, config=assocparser.AssocParserConfig()):
         """
         Arguments:
         ---------
@@ -20,7 +20,7 @@ class HpoaParser(AssocParser):
         """
 
         self.config = config
-        self.report = assocparser.Report()
+        self.report = assocparser.Report(config=self.config)
 
     def skim(self, file):
         file = self._ensure_file(file)
@@ -40,13 +40,13 @@ class HpoaParser(AssocParser):
             if self._is_exclude_relation(relation):
                 continue
             id = self._pair_to_id(vals[0], vals[1])
-            if not self._validate_id(id, line, ENTITY):
+            if not self._validate_id(id, line, context=ENTITY):
                 continue
             n = vals[2]
             t = vals[4]
             tuples.append( (id,n,t) )
         return tuples
-    
+
     def parse_line(self, line):
         """
         Parses a single line of a HPOA file
@@ -98,6 +98,8 @@ class HpoaParser(AssocParser):
 
         # hardcode this, as HPOA is currently human-only
         taxon = 'NCBITaxon:9606'
+        split_line = assocparser.SplitLine(line=line, values=vals, taxon=taxon)
+
 
         # hardcode this, as HPOA is currently disease-only
         db_object_type = 'disease'
@@ -106,12 +108,17 @@ class HpoaParser(AssocParser):
         ## db + db_object_id. CARD=1
         ## --
         id = self._pair_to_id(db, db_object_id)
-        if not self._validate_id(id, line, ENTITY):
+        if not self._validate_id(id, split_line, context=ENTITY):
             return assocparser.ParseResult(line, [], True)
 
-        if not self._validate_id(hpoid, line, ANNOTATION):
+        if not self._validate_id(hpoid, split_line, context=ANNOTATION):
             return assocparser.ParseResult(line, [], True)
 
+        valid_hpoid = self._validate_ontology_class_id(hpoid, split_line)
+        if valid_hpoid == None:
+            return assocparser.ParseResult(line, [], True)
+        hpoid = valid_hpoid
+
         # validation
         #self._validate_symbol(db_object_symbol, line)
 
@@ -166,6 +173,12 @@ class HpoaParser(AssocParser):
             else:
                 relation = None
 
+        # With/From
+        withfroms = self.validate_pipe_separated_ids(withfrom, split_line, empty_allowed=True, extra_delims=",")
+        if withfroms == None:
+            # Reporting occurs in above function call
+            return assocparser.ParseResult(line, [], True)
+
         ## --
         ## hpoid
         ## --
@@ -191,7 +204,7 @@ class HpoaParser(AssocParser):
         evidence = {
             'type': evidence,
             'has_supporting_reference': reference.split("; "),
-            'with_support_from': self._split_pipe(withfrom)
+            'with_support_from': withfroms
         }
 
         ## Construct main return dict
@@ -204,14 +217,13 @@ class HpoaParser(AssocParser):
             'relation': {
                 'id': relation
             },
+            'interacting_taxon': None,
             'evidence': evidence,
             'provided_by': assigned_by,
             'date': date,
 
         }
 
-        self._validate_assoc(assoc, line)
-
         return assocparser.ParseResult(line, [assoc], False)
 
     def is_header(self, line):
