@@ -7,10 +7,12 @@ import logging
 
 from dataclasses import dataclass
 
-from typing import List, Dict, Any, Tuple, Union
+from typing import List, Dict, Any, Optional, Set, Tuple, Union
+from prefixcommons import curie_util
+from requests.models import REDIRECT_STATI
 from ontobio import ontol
 from ontobio import ecomap
-from ontobio.io import assocparser
+from ontobio.io import assocparser, gafparser
 from ontobio.io import gaference
 from ontobio.model import association
 from ontobio.rdfgen import relations
@@ -21,18 +23,37 @@ FailMode = enum.Enum("FailMode", {"SOFT": "soft", "HARD": "hard"})
 ResultType = enum.Enum("Result", {"PASS": "Pass", "WARNING": "Warning", "ERROR": "Error"})
 RepairState = enum.Enum("RepairState", {"OKAY": "Okay", "REPAIRED": "Repaired", "FAILED": "Failed"})
 
+
 # TestResult = collections.namedtuple("TestResult", ["result_type", "message", "result"])
 class TestResult(object):
-    def __init__(self, result_type: ResultType, message: str, result: List):
+    """
+    Represents the result of a single association.GoAssociation being validated on some rule
+    """
+    def __init__(self, result_type: ResultType, message: str, result):
+        """
+        Create a new TestResult
+
+        Args:
+            result_type (ResultType): enum of PASS, WARNING, ERROR. Both WARNINGs and ERRORs are reported, but ERROR will filter the offending GoAssociation
+            message (str): Description of the failure of GoAssociation to pass a rule. This is usually just the rule title
+            result: [description] True if the GoAssociation passes, False if not. If it's repaired, this is the updated, repaired, GoAssociation
+        """
         self.result_type = result_type
         self.message = message
         self.result = result
 
+
 """
 Send True for passes, and this returns the PASS ResultType, and if False, then
 depending on the fail mode it returns either WARNING or ERROR ResultType.
 """
+
+
 def result(passes: bool, fail_mode: FailMode) -> ResultType:
+    """
+    Send True for passes, and this returns the PASS ResultType, and if False, then
+    depending on the fail mode it returns either WARNING or ERROR ResultType.
+    """
     if passes:
         return ResultType.PASS
 
@@ -43,7 +64,20 @@ def result(passes: bool, fail_mode: FailMode) -> ResultType:
     if fail_mode == FailMode.HARD:
         return ResultType.ERROR
 
+
 def repair_result(repair_state: RepairState, fail_mode: FailMode) -> ResultType:
+    """
+    Returns ResultType.PASS if the repair_state is OKAY, and WARNING if REPAIRED.
+
+    This is used by RepairRule implementations.
+
+    Args:
+        repair_state (RepairState): If the GoAssocition was repaired during a rule, then this should be RepairState.REPAIRED, otherwise RepairState.OKAY
+        fail_mode (FailMode): [description]
+
+    Returns:
+        ResultType: [description]
+    """
     if repair_state == RepairState.OKAY:
         return ResultType.PASS
 
@@ -76,6 +110,12 @@ class GoRule(object):
         return len(self.run_context_tags) == 0 or any(self.run_context_tags & rule_tags_to_match)
 
     def _run_if_context(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
+        if not config.rule_set.should_run_rule(self.id.split(":")[1]):
+            # We check that this rule can be run based on the ID and what is in the config rule_set.
+            # If we should not run the rule, we'll auto-pass it here.
+            # In the future, we could use a new result type, SKIP here.
+            return TestResult(ResultType.PASS, "", annotation)
+
         result = TestResult(ResultType.PASS, "", annotation)
         if self._is_run_from_context(config):
             result = self.test(annotation, config, group=group)
@@ -87,11 +127,21 @@ class GoRule(object):
         result.result = annotation
         return result
 
-
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
-        # run_test() -> _run_if_context() -> test()
+        """
+        Subclasses should override this function to implement the logic of the rule.
+
+        Args:
+            annotation (association.GoAssociation): The incoming annotation under test
+            config (assocparser.AssocParserConfig): The configuration object defined for this execution of the rules. This should stay more or less constant across each rule test run as well as across each annotation being passed into the rules engine. This holds things like the ontology, etc.
+            group ([type], optional): The name of the upstream resource group whose annotation is being tested. This may be None.
+
+        Returns:
+            TestResult: See documentation for TestResult above
+        """
         pass
 
+
 class RepairRule(GoRule):
 
     def __init__(self, id, title, fail_mode, tags=[]):
@@ -119,12 +169,12 @@ class GoRule02(GoRule):
     def __init__(self):
         super().__init__("GORULE:0000002", "No 'NOT' annotations to 'protein binding ; GO:0005515'", FailMode.SOFT)
 
-
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
 
-        fails = (annotation.object.id == "GO:0005515" and annotation.negated)
+        fails = (str(annotation.object.id) == "GO:0005515" and annotation.negated)
         return self._result(not fails)
 
+
 class GoRule06(GoRule):
 
     def __init__(self):
@@ -136,11 +186,12 @@ class GoRule06(GoRule):
         if config.ontology is None:
             return self._result(True)
 
-        go_namespace = [predval for predval in config.ontology.get_graph().node.get(annotation.object.id, {}).get("meta", {}).get("basicPropertyValues", []) if predval["pred"]=="OIO:hasOBONamespace"]
-        evidence = annotation.evidence.type
+        go_namespace = [predval for predval in config.ontology.get_graph().nodes.get(str(annotation.object.id), {}).get("meta", {}).get("basicPropertyValues", []) if predval["pred"]=="OIO:hasOBONamespace"]
+        evidence = str(annotation.evidence.type)
         fails = evidence in [self.iep, self.hep] and "biological_process" not in [o["val"] for o in go_namespace]
         return self._result(not fails)
 
+
 class GoRule07(GoRule):
 
     def __init__(self):
@@ -153,8 +204,8 @@ class GoRule07(GoRule):
             # We'll define children_of_catalytic_activity if we have an ontology *and* if we haven't defined it before already
             self.children_of_catalytic_activity = set(config.ontology.descendants(catalytic_activity, relations=["subClassOf"], reflexive=True))
 
-        goterm = annotation.object.id
-        evidence = annotation.evidence.type
+        goterm = str(annotation.object.id)
+        evidence = str(annotation.evidence.type)
 
         fails = False
         if self.children_of_catalytic_activity is not None:
@@ -183,8 +234,8 @@ class GoRule08(GoRule):
             self.do_not_annotate = []
             self.do_not_manually_annotate = []
 
-        goid = annotation.object.id
-        evidence = annotation.evidence.type
+        goid = str(annotation.object.id)
+        evidence = str(annotation.evidence.type)
 
         auto_annotated = goid in self.do_not_annotate
         manually_annotated = evidence != "ECO:0000501" and goid in self.do_not_manually_annotate
@@ -202,14 +253,15 @@ class GoRule11(GoRule):
         self.nd = "ECO:0000307"
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
-        goclass = annotation.object.id
-        evidence = annotation.evidence.type
+        goclass = str(annotation.object.id)
+        evidence = str(annotation.evidence.type)
 
         # If we see a bad evidence, and we're not in a paint file then fail.
         # We're good if both predicates are true, or neither are true
         success = (evidence == self.nd and goclass in self.root_go_classes) or (evidence != self.nd and goclass not in self.root_go_classes)
         return self._result(success)
 
+
 class GoRule13(GoRule):
 
     def __init__(self):
@@ -236,12 +288,13 @@ class GoRule13(GoRule):
             return self._result(True)
         else:
             # Filter non experimental evidence
-            if annotation.evidence.type in self.non_experimental_evidence:
+            if str(annotation.evidence.type) in self.non_experimental_evidence:
                 return self._result(False)
             else:
                 # Only submit a warning/report if we are an experimental evidence
                 return TestResult(ResultType.WARNING, self.title, False)
 
+
 class GoRule15(GoRule):
 
     def __init__(self):
@@ -252,7 +305,7 @@ class GoRule15(GoRule):
 
         # Cache the allowed terms
         if self.allowed_dual_species_terms is None and config.ontology is not None:
-            interaction_terms = config.ontology.descendants("GO:0044419", relations=["subClassOf"], reflexive=True)
+            interaction_terms = config.ontology.descendants("GO:0044419", relations=["subClassOf", "BFO:0000050"], reflexive=True)
             interspecies_interactions_regulation = config.ontology.descendants("GO:0043903", relations=["subClassOf"], reflexive=True)
             host_cellular_component = config.ontology.descendants("GO:0018995", relations=["subClassOf"], reflexive=True)
             self.allowed_dual_species_terms = set(interaction_terms + interspecies_interactions_regulation + host_cellular_component)
@@ -262,7 +315,7 @@ class GoRule15(GoRule):
         passes = False
         if self.allowed_dual_species_terms is not None:
             dual = annotation.interacting_taxon is not None
-            goterm = annotation.object.id
+            goterm = str(annotation.object.id)
 
             # We fail if we are a dual taxon and then the term is not in this list
             # This is the same as dual -> goterm in list
@@ -278,12 +331,12 @@ class GoRule16(GoRule):
         super().__init__("GORULE:0000016", "All IC annotations should include a GO ID in the \"With/From\" column", FailMode.HARD)
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
-        evidence = annotation.evidence.type
+        evidence = str(annotation.evidence.type)
         withfrom = annotation.evidence.with_support_from
 
         okay = True
         if evidence == "ECO:0000305":
-            only_go = [t for conjunctions in withfrom for t in conjunctions.elements if t.startswith("GO:")] # Filter terms that aren't GO terms
+            only_go = [t for conjunctions in withfrom for t in conjunctions.elements if t.namespace == "GO"] # Filter terms that aren't GO terms
             okay = len(only_go) >= 1
 
         return self._result(okay)
@@ -295,7 +348,7 @@ class GoRule17(GoRule):
         super().__init__("GORULE:0000017", "IDA annotations must not have a With/From entry", FailMode.SOFT)
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
-        evidence = annotation.evidence.type
+        evidence = str(annotation.evidence.type)
         withfrom = annotation.evidence.with_support_from
 
         if evidence == "ECO:0000314":
@@ -303,13 +356,14 @@ class GoRule17(GoRule):
         else:
             return self._result(True)
 
+
 class GoRule18(GoRule):
 
     def __init__(self):
         super().__init__("GORULE:0000018", "IPI annotations require a With/From entry", FailMode.SOFT)
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
-        evidence = annotation.evidence.type
+        evidence = str(annotation.evidence.type)
         withfrom = annotation.evidence.with_support_from
 
         if evidence == "ECO:0000353":
@@ -325,11 +379,12 @@ class GoRule26(GoRule):
         self.offending_evidence = ["ECO:0000318"]
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
-        evidence = annotation.evidence.type
+        evidence = str(annotation.evidence.type)
         # If we see a bad evidence, and we're not in a paint file then fail.
         fails = (evidence in self.offending_evidence and not config.paint)
         return self._result(not fails)
 
+
 class GoRule28(RepairRule):
     def __init__(self):
         super().__init__("GORULE:0000028", "Aspect can only be one of C, P, F", FailMode.HARD)
@@ -341,12 +396,12 @@ class GoRule28(RepairRule):
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
         aspect = annotation.aspect
-        goterm = annotation.object.id
+        goterm = str(annotation.object.id)
 
         if config.ontology is None:
             return TestResult(ResultType.PASS, self.title, annotation)
 
-        namespaces = [predval for predval in config.ontology.get_graph().node.get(goterm, {}).get("meta", {}).get("basicPropertyValues", []) if predval["pred"]=="OIO:hasOBONamespace"]
+        namespaces = [predval for predval in config.ontology.get_graph().nodes.get(goterm, {}).get("meta", {}).get("basicPropertyValues", []) if predval["pred"]=="OIO:hasOBONamespace"]
         # the namespaces expression cascades through the json representation of this
         # ontology using empty dict/list if the key is not present
 
@@ -377,16 +432,16 @@ class GoRule29(GoRule):
         self.two_years = datetime.timedelta(days=730)
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
-        evidence = annotation.evidence.type
+        evidence = str(annotation.evidence.type)
         date = annotation.date
 
         now = datetime.datetime.today()
 
         time_compare_delta_short = self.one_year
         time_compare_delta_long = self.two_years
-        time_diff = now - datetime.datetime(int(date[0:4]),
-                                            int(date[4:6]),
-                                            int(date[6:8]),
+        time_diff = now - datetime.datetime(int(date.year),
+                                            int(date.month),
+                                            int(date.day),
                                             0, 0, 0, 0)
 
         iea = "ECO:0000501"
@@ -399,6 +454,7 @@ class GoRule29(GoRule):
         ## Default results we we get here.
         return self._result(True)
 
+
 class GoRule30(GoRule):
 
     def __init__(self):
@@ -413,12 +469,14 @@ class GoRule30(GoRule):
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
         references = annotation.evidence.has_supporting_reference
         for ref in references:
+            ref = str(ref)
             # Not allowed is obsolete and GO_PAINT:x
             if ref.startswith("GO_PAINT") or (config.goref_metadata is not None and config.goref_metadata.get(self._ref_curi_to_id(ref), {}).get("is_obsolete", False)):
                 return self._result(False)
 
         return self._result(True)
 
+
 class GoRule37(GoRule):
 
     def __init__(self):
@@ -426,8 +484,8 @@ class GoRule37(GoRule):
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
         # If the evidence code is IBA, then (1) the assigned_by field must be GO_Central and (2) the reference field must be PMID:21873635
-        evidence = annotation.evidence.type
-        references = annotation.evidence.has_supporting_reference
+        evidence = str(annotation.evidence.type)
+        references = [str(ref) for ref in annotation.evidence.has_supporting_reference]
         assigned_by = annotation.provided_by
 
         result = self._result(True) # By default we pass
@@ -436,6 +494,7 @@ class GoRule37(GoRule):
 
         return result
 
+
 class GoRule39(GoRule):
 
     def __init__(self):
@@ -444,19 +503,20 @@ class GoRule39(GoRule):
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
         # An implementation note: This is done by testing if the DB (column 1) is ComplexPortal.
         # This will grab a subset of all actual Protein Complexes. This is noted in the rule description
-        db = annotation.subject.id.split(":")[0]
-        goterm = annotation.object.id
+        db = annotation.subject.id.namespace
+        goterm = str(annotation.object.id)
 
         fails = (db == "ComplexPortal" and goterm == "GO:0032991")
         return self._result(not fails)
 
+
 class GoRule42(GoRule):
 
     def __init__(self):
         super().__init__("GORULE:0000042", "Qualifier: IKR evidence code requires a NOT qualifier", FailMode.HARD)
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
-        evidence = annotation.evidence.type
+        evidence = str(annotation.evidence.type)
 
         result = self._result(True)
         ikr = "ECO:0000320"
@@ -465,6 +525,7 @@ class GoRule42(GoRule):
 
         return result
 
+
 class GoRule43(GoRule):
 
     def __init__(self):
@@ -481,8 +542,8 @@ class GoRule43(GoRule):
         if config.goref_metadata is None:
             return self._result(True)
 
-        references = annotation.evidence.has_supporting_reference
-        evidence = annotation.evidence.type
+        references = [str(ref) for ref in annotation.evidence.has_supporting_reference]
+        evidence = str(annotation.evidence.type)
 
         for ref in references:
             allowed_eco = config.goref_metadata.get(self._ref_curi_to_id(ref), {}).get("evidence_codes", None)
@@ -516,7 +577,7 @@ class GoRule46(GoRule):
             self.self_binding_terms = self.self_binding_roots
 
         withfroms = annotation.evidence.with_support_from
-        goterm = annotation.object.id
+        goterm = str(annotation.object.id)
 
         if goterm in self.self_binding_terms:
             # Then we're in the self-binding case, and check if object ID is in withfrom
@@ -528,6 +589,7 @@ class GoRule46(GoRule):
 
         return self._result(True)
 
+
 class GoRule50(GoRule):
 
     def __init__(self):
@@ -535,8 +597,9 @@ class GoRule50(GoRule):
         self.the_evidences = ["ECO:0000250", "ECO:0000247", "ECO:0000266"]
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
-        # should not have the same identifier in the 'gene product column' (column 2) and in the 'with/from' column (column 8)
-        evidence = annotation.evidence.type
+        # should not have the same identifier in the 'gene product column' (column 2) and in the 'with/from' column
+        # (column 8)
+        evidence = str(annotation.evidence.type)
         result = self._result(True)
         if evidence in self.the_evidences:
             # Ensure the gp ID is not an entry in withfrom
@@ -545,6 +608,23 @@ class GoRule50(GoRule):
 
         return result
 
+
+class GoRule55(GoRule):
+
+    def __init__(self):
+        super().__init__("GORULE:0000055", "References should have only one ID per ID space", FailMode.SOFT)
+
+    def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
+        found_id_spaces = dict()
+        for ref in annotation.evidence.has_supporting_reference:
+            if ref.namespace in found_id_spaces:
+                return self._result(False)
+            else:
+                found_id_spaces[ref.namespace] = ref
+        # We found no duplicate IDs, so we good
+        return self._result(True)
+
+
 class GoRule57(GoRule):
 
     def __init__(self):
@@ -556,14 +636,14 @@ class GoRule57(GoRule):
             return self._result(True)
 
         evidence_codes = config.group_metadata.get("filter_out", {}).get("evidence", [])
-        if annotation.evidence.type in evidence_codes:
+        if str(annotation.evidence.type) in evidence_codes:
             return self._result(False)
 
         evidences_references = config.group_metadata.get("filter_out", {}).get("evidence_reference", [])
         for er in evidences_references:
             evidence_code = er["evidence"]
             reference = er["reference"]
-            if annotation.evidence.type == evidence_code and annotation.evidence.has_supporting_reference == reference:
+            if str(annotation.evidence.type) == evidence_code and [str(ref) for ref in annotation.evidence.has_supporting_reference] == [reference]:
                 return self._result(False)
 
         properties = config.group_metadata.get("filter_out", {}).get("annotation_properties", [])
@@ -573,10 +653,12 @@ class GoRule57(GoRule):
 
         return self._result(True)
 
+
 class GoRule58(RepairRule):
 
     def __init__(self):
-        super().__init__("GORULE:0000058", "Object extensions should conform to the extensions-patterns.yaml file in metadata", FailMode.HARD, tags=["context-import"])
+        super().__init__("GORULE:0000058", "Object extensions should conform to the extensions-patterns.yaml "
+                                           "file in metadata", FailMode.HARD, tags=["context-import"])
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
 
@@ -591,7 +673,7 @@ class GoRule58(RepairRule):
         bad_conjunctions = []
         for con in annotation.object_extensions:
             # Count each extension unit, represented by tuple (Relation, Namespace)
-            extension_counts = collections.Counter([(unit.relation, unit.term.split(":")[0]) for unit in con.elements])
+            extension_counts = collections.Counter([(str(unit.relation), unit.term.namespace) for unit in con.elements])
 
             matches = self._do_conjunctions_match_constraint(con, annotation.object.id, config.extensions_constraints, extension_counts)
             # If there is a match in the constraints, then we're all good and we can exit with a pass!
@@ -625,10 +707,11 @@ class GoRule58(RepairRule):
 
             extension_good = False
             for constraint in constraints:
+                constraint_relation_uri = association.relations.lookup_label(constraint["relation"])
+                ext_relation_uri = curie_util.expand_uri(str(ext.relation))
+                if ext_relation_uri == constraint_relation_uri:
 
-                if ext.relation == constraint["relation"]:
-
-                    if (ext.term.split(":")[0] in constraint["namespaces"] and term in constraint["primary_terms"]):
+                    if (ext.term.namespace in constraint["namespaces"] and str(term) in constraint["primary_terms"]):
                         # If we match namespace and go term, then if we there is a cardinality constraint, check that.
                         if "cardinality" in constraint:
                             cardinality_violations = [(ext, num) for ext, num in dict(conjunction_counts).items() if num > constraint["cardinality"]]
@@ -649,6 +732,124 @@ class GoRule58(RepairRule):
         return True
 
 
+class GoRule61(RepairRule):
+
+    def __init__(self):
+        super().__init__("GORULE:0000061", "Only certain gene product to term relations are allowed for a given GO term", FailMode.HARD)
+        self.protein_containing_complex_descendents = None
+
+        self.allowed_mf = set([association.Curie(namespace="RO", identity="0002327"), association.Curie(namespace="RO", identity="0002326")])
+        self.allowed_bp = set([association.Curie("RO", "0002331"), association.Curie("RO", "0002264"),
+                            association.Curie("RO", "0004032"), association.Curie("RO", "0004033"), association.Curie("RO", "0002263"),
+                            association.Curie("RO", "0004034"), association.Curie("RO", "0004035")])
+        self.allowed_cc_complex = set([association.Curie("BFO", "0000050")])
+        self.repairable_cc_complex = set([association.Curie("RO", "0002432"), association.Curie("RO", "0001025")])
+        self.allowed_cc_other = set([association.Curie("RO", "0001025"), association.Curie("RO", "0002432"), association.Curie("RO", "0002325")])
+
+    def make_protein_complex_descendents_if_not_present(self, ontology: Optional[ontol.Ontology]) -> Set:
+        if ontology is not None and self.protein_containing_complex_descendents is None:
+            closure = gafparser.protein_complex_sublcass_closure(ontology)
+            self.protein_containing_complex_descendents = closure
+
+        return {} if self.protein_containing_complex_descendents is None else self.protein_containing_complex_descendents
+
+    def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
+        """
+        * GO:0003674 "molecular function"
+            * Term: GO:0005554 => relation is RO:0002327 "enables" + repair,
+            * Term: subclass of GO:0005554 => relations: {RO:0002327 "enables", RO:0002326 "contributes_to"} + filter
+        * GO:0008150 "biological process"
+            * Term: GO:0008150 => RO:0002331 "involved_in" + repair
+            * Term: subclass of GO:0008150 => relations: {RO:0002331 "involved_in", RO:0002264 "acts upstream or within", RO:0004032 "acts upstream of or within, positive effect", RO:0004033 "acts upstream of or within, negative effect", RO:0002263 "acts upstream of", RO:0004034 "acts upstream of, positive effect", RO:0004035 "acts upstream of, negative effect"} + filter
+        * GO:0005575 "cellular component"
+            * Term: GO:0005575 => relation is RO:0002432 "is_active_in" + repair
+            * If term is subclass of `GO:0032991 "protein-containing complex"` with relation one of {RO:0002432 "is_active_in", RO:0001025 "located in"} => relation should be repaired to `BFO:0000050 "part of"`
+            * If term is subclass of `GO:0032991` and any other relation, then it should be filtered
+            * Term: any other subclass of `GO:0008372` => allowed relations are {`RO:0001025 "located in"`, `RO:0002432 "is_active_in"`, `RO:0002325 "colocalizes_with"`} and other relations repaired to `RO:0001025 "located in"`.
+        """
+        if config.ontology is None:
+            return TestResult(ResultType.PASS, "", annotation)
+
+        term = str(annotation.object.id)
+        namespace = config.ontology.obo_namespace(term)
+        repair_state = RepairState.OKAY
+        relation = annotation.relation
+        allowed = set()
+
+        repaired_annotation = annotation
+        if term == "GO:0005554":
+            enables = association.Curie(namespace="RO", identity="0002327")
+            if relation != enables:
+                repaired_annotation = copy.deepcopy(annotation)
+                repaired_annotation.relation = enables
+                repaired_annotation.qualifiers = [enables]
+                allowed = set([enables])
+                repair_state = RepairState.REPAIRED
+        elif namespace == "molecular_function":
+            if relation not in self.allowed_mf:
+                enables = association.Curie(namespace="RO", identity="0002327")
+                repaired_annotation = copy.deepcopy(annotation)
+                repaired_annotation.relation = enables
+                repaired_annotation.qualifiers = [enables]
+                allowed = self.allowed_mf
+                repair_state = RepairState.REPAIRED
+        elif term == "GO:0008150":
+            involved_in = association.Curie(namespace="RO", identity="0002331")
+            if relation != involved_in:
+                repaired_annotation = copy.deepcopy(annotation)
+                repaired_annotation.relation = involved_in
+                repaired_annotation.qualifiers = [involved_in]
+                allowed = set([involved_in])
+                repair_state = RepairState.REPAIRED
+        elif namespace == "biological_process":
+            acts_upstream_of_or_within = association.Curie("RO", "0002264")
+            if relation not in self.allowed_bp:
+                repaired_annotation = copy.deepcopy(annotation)
+                repaired_annotation.relation = acts_upstream_of_or_within
+                repaired_annotation.qualifiers = [acts_upstream_of_or_within]
+                allowed = self.allowed_bp
+                repair_state = RepairState.REPAIRED
+        elif term == "GO:0005575":
+            is_active_in = association.Curie(namespace="RO", identity="0002432")
+            if relation != is_active_in:
+                repaired_annotation = copy.deepcopy(annotation)
+                repaired_annotation.relation = is_active_in
+                repaired_annotation.qualifiers = [is_active_in]
+                allowed = set([is_active_in])
+                repair_state = RepairState.REPAIRED
+        elif namespace == "cellular_component":
+            if term in self.make_protein_complex_descendents_if_not_present(config.ontology):
+                part_of = association.Curie(namespace="BFO", identity="0000050")
+                if relation not in self.allowed_cc_complex:
+                    if relation in self.repairable_cc_complex:
+                        repaired_annotation = copy.deepcopy(annotation)
+                        repaired_annotation.relation = part_of
+                        repaired_annotation.qualifiers = [part_of]
+                        allowed = self.allowed_cc_complex
+                        repair_state = RepairState.REPAIRED
+                    else:
+                        # Not repairable to part_of, so filter
+                        repaired_annotation = annotation
+                        allowed = self.allowed_cc_complex
+                        repair_state = RepairState.FAILED
+            else:
+                located_in = association.Curie(namespace="RO", identity="0001025")
+                if relation not in self.allowed_cc_other:
+                    repaired_annotation = copy.deepcopy(annotation)
+                    repaired_annotation.relation = located_in
+                    repaired_annotation.qualifiers = [located_in]
+                    allowed = self.allowed_cc_other
+                    repair_state = RepairState.REPAIRED
+        else:
+            # If we reach here, we're in a weird case where a term is not in either
+            # of the three main GO branches, or does not have a namespace defined.
+            # If this is the case we should just pass along as if the ontology is missing
+            return TestResult(repair_result(RepairState.OKAY, self.fail_mode), "{}: {}".format(self.message(repair_state), "GO term has no namespace"), repaired_annotation)
+
+        allowed_str = ", ".join([str(a) for a in allowed])
+        return TestResult(repair_result(repair_state, self.fail_mode), "{}: {} should be one of {}".format(self.message(repair_state), relation, allowed_str), repaired_annotation)
+
+
 GoRules = enum.Enum("GoRules", {
     "GoRule02": GoRule02(),
     "GoRule06": GoRule06(),
@@ -669,14 +870,18 @@ GoRules = enum.Enum("GoRules", {
     "GoRule43": GoRule43(),
     "GoRule46": GoRule46(),
     "GoRule50": GoRule50(),
+    "GoRule55": GoRule55(),
     "GoRule57": GoRule57(),
     "GoRule58": GoRule58(),
+    "GoRule61": GoRule61(),
     # GoRule13 at the bottom in order to make all other rules clean up an annotation before reaching 13
     "GoRule13": GoRule13()
 })
 
 
 GoRulesResults = collections.namedtuple("GoRulesResults", ["all_results", "annotation"])
+
+
 def test_go_rules(annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> GoRulesResults:
     all_results = {}
 
